function [dist, matrix] = CDTW(Q, S, Lnorm, r)
%Computes the unnormalized constrained DTW distance between Q and S
%#eml
% Note the Sakoe-Chiba band width r is defined with respect to Q, meaning
% that if |S| > |Q|, than the band width w.r.t. S is larger than r. And
% vice versa if |S| < |Q| the band width w.r.t. S is smaller than r.

%% Init
n = size(Q,1);
m = size(S,1);
matrix = zeros(n+1,m+1);
w = n/m;

%% Compute constraints
jmin = zeros(n,1);
jmax = zeros(n,1);
for i = 1:n
    jmin(i) = max(1,ceil((i-r)/w));
    jmax(i) = min(m,floor((i+r)/w));
end

%% Mark Infinities
for i = 1:n
    matrix(i+1,jmin(i)) = Inf;
end
for j = 1:m
    matrix(max(1,ceil(j*w-r)),j+1) = Inf;
end

%% Compute DTW
for i = 2:n+1
    im1 = i-1;
    for j = (jmin(im1)+1):(jmax(im1)+1);
        jm1 = j-1;
        matrix(i,j) = sum(abs((Q(im1,:)-S(jm1,:)).^Lnorm)) + ...
                          min(matrix(im1,j), ...
                              min(matrix(i,jm1),matrix(im1,jm1)));
    end
end

dist = matrix(n+1,m+1);

end