function [dist, matrix] = DTW(Q, S, Lnorm)
%Computes the unnormalized dynamic time-warping distance between Q and S
%#eml

%% Init
n = size(Q,1);
m = size(S,1);
matrix = zeros(n+1,m+1);

%% Mark Infinities
for i = 2:n+1
    matrix(i,1) = Inf;
end
for j = 2:m+1
    matrix(1,j) = Inf;
end

%% Compute DTW
for i = 2:n+1
    im1 = i-1;
    for j = 2:m+1
        jm1 = j-1;
        matrix(i,j) = sum(abs((Q(im1,:)-S(jm1,:)).^Lnorm)) + ...
                          min(matrix(im1,j), ...
                              min(matrix(i,jm1),matrix(im1,jm1)));
    end
end

dist = matrix(n+1,m+1);

end