function [U, L] = envelopes(Q, r)
%Computes envelopes of queries for LB_Keogh speedup

%% Init
n = size(Q,1);
U = cell(n,1);
L = cell(n,1);

[m,d] = size(Q{1}); % all queries are expected to be of equal size

%% Compute constraints
jmin = zeros(m,1);
jmax = zeros(m,1);
for i = 1:m
    jmin(i) = max(1,ceil(i-r));
    jmax(i) = min(m,floor(i+r));
end

%% Compute envelopes
for i = 1:n
    query = Q{i};
    u = zeros(m,d);
    l = zeros(m,d);
    for j = 1:m
        inrange = query(jmin(j):jmax(j),:);
        u(j,:) = max(inrange,[],1);
        l(j,:) = min(inrange,[],1);
    end
    U{i} = u;
    L{i} = l;
end

end